#ifndef _included_sfcIndex_h
#define _included_sfcIndex_h

/*
*************************************************************************
*                                                                       *
* sfcIndex.h                                                            *
*                                                                       *
* Class sfcIndex defines an index generated by the chosen SFC mapping	*
* i.e. specified space dimensions, base used and mapping function. 	*
* sfcIndex is an ordering vector of sfcIndexUnits; each sfcIndexUnits 	*
* defined as Dim Base-b digts, and defining the index at each level of  *
* the sfc. The most-significant sfcIndexUnits corresponds to level 1	*
* Class sfcIndex is derived from BitVec.                                *
*                                                                       *
* Author:  Manish Parashar (parashar@cs.utexas.edu)                     *
*                                                                       *
*************************************************************************
*/

#include "BitVec.h"
#include <math.h>

#define log2(x) (log(x))/(log(2.0))

#ifndef sfcBase
#define sfcBase (2)
#endif

class sfcIndex : public BitVec
  {
   friend ostream& operator<<(ostream&, sfcIndex const &);

   short int sfcbasew;
   short int sfcdigitw;

protected:
   short int sfcbase;
   short int sfcdim;
   short int sfclevs;

public:
   inline sfcIndex()
	: BitVec(),
	  sfcdim(0), sfclevs(0), sfcbase(0),
	  sfcbasew(0), sfcdigitw(0) {} 

   sfcIndex(const int dim, const int levs, const int base = sfcBase);

   inline sfcIndex(sfcIndex const &other)
	: BitVec((BitVec const &)other),
          sfcdim(other.sfcdim), sfclevs(other.sfclevs),
	  sfcbase(other.sfcbase), sfcbasew(other.sfcbasew),
          sfcdigitw(other.sfcdigitw) {}

   sfcIndex const &operator= (sfcIndex const &other);

   int operator== (sfcIndex const &other) const;
   int operator!= (sfcIndex const &other) const;
   int operator> (sfcIndex const &other) const;
   int operator< (sfcIndex const &other) const;
   int operator>= (sfcIndex const &other) const;
   int operator<= (sfcIndex const &other) const;

   /* Operations on digits; i.e. (sfcdim*sfcbasew) bits */
   unsigned sfcGetDigit(const int level) const;
   void sfcSetDigit(const int level, const unsigned digit);
   void sfcUnsetDigit(const int level);
   void sfcMaxDigit(const int level);

   /* Operations on base sfcbase words (sfcbasew bits) within a digit */
   void sfcSwapDigitWd(const int level, const int pos1, const int pos2);
   void sfcToggleDigitWd(const int level, const int pos);

   /* Operations on the sfcIndex */
   unsigned sfcGetIndex() const;
   unsigned sfcGetIndex(const int lev) const;
   unsigned sfcGetIndex(const int lev, const int nlev) const;

   void sfcSetIndex(const unsigned index);
   void sfcSetIndex(const unsigned index, const int lev);
   void sfcSetIndex(const unsigned index, const int lev, const int nlev);
   void sfcMaxIndex(int const lev);

   /* Mapping, inverse mapping routines */
   void sfcCreateLocNum(unsigned const *coords);
   void sfcGetCoords(unsigned *coords) const;
   void sfcGetDigitArray(unsigned *digits) const;

   /* Query interface */
   inline int sfcGetBaseW(void) const
	{return sfcbasew;}
   inline int sfcGetDigitW(void) const
	{return sfcdigitw;}

private:
   inline void sfcSetDigitBit(const int level, const int pos)
	{ SetBit((level-1)*sfcdigitw + pos); }
   inline void sfcUnsetDigitBit(const int level, const int pos)
	{ UnsetBit((level-1)*sfcdigitw + pos); }
   inline void sfcToggleDigitBit(const int level, const int pos)
	{ ToggleBit((level-1)*sfcdigitw + pos); }

  };
 
#ifdef __ENABLE_INLINE__
#include "sfcIndex.inline"
#endif

extern ostream&  operator << (ostream& os, sfcIndex const &si);

#endif
